@@ -152,6 +152,9 @@ static int filter_nopar_gen[] = {
     SCMP_SYS(close),
     SCMP_SYS(clone),
     SCMP_SYS(dup),
+#ifdef __NR_clone3
+    SCMP_SYS(clone3),
+#endif
     SCMP_SYS(epoll_create),
     SCMP_SYS(epoll_wait),
 #ifdef __NR_epoll_pwait
@@ -224,6 +227,9 @@ static int filter_nopar_gen[] = {
 #endif
     SCMP_SYS(read),
     SCMP_SYS(rt_sigreturn),
+#ifdef __NR_rseq
+    SCMP_SYS(rseq),
+#endif
     SCMP_SYS(sched_getaffinity),
 #ifdef __NR_sched_yield
     SCMP_SYS(sched_yield),
